/**
 * Ephoto Dam Addon for Drupal 9
 *
 * @base EPHOTO DAM FIELD
 * @author EINDEN
 */

(function ($) {

Drupal.ephotoDamField = function(name, settings) {
    /**
     * Ephoto Dam ID of Drupal Addon
     */		
    this.CLIENT_ID = 'CaCvqS5H';

    /**
     * Name of field
     */
    this._name = name;
    
    /**
     * Settings
     */    
    this._settings = settings;

    /**
     * Current authid
     */     
    this._authid = null;

    /**
     * Ephoto Dam instance
     */		
    this._ephoto = null;
    
    /**
     * Ephoto Dam Server connected
     */		
    this._connected = false;
    
    /**
     * Current selected field
     */
    this._fieldSelected = null;
};

Drupal.ephotoDamField.prototype = {
    /**
     * Enable version support (from default values form)
     * 
     * @param {node} el
     */    
    enableVersionSupport: function(el) {
        if (el.checked) {
            $('.ephoto-dam-field-version-field').removeClass('ephoto-dam-field-hidden');
            $('.ephoto-dam-field-version-prefix').removeClass('ephoto-dam-field-hidden');
        
            $('.ephoto-dam-field-versions-button').removeClass('ephoto-dam-field-hidden');
            $('.ephoto-dam-field-file-button').addClass('ephoto-dam-field-hidden');
        }

        else {
            $('.ephoto-dam-field-version-field').addClass('ephoto-dam-field-hidden');
            $('.ephoto-dam-field-version-prefix').addClass('ephoto-dam-field-hidden');        
        
            $('.ephoto-dam-field-versions-button').addClass('ephoto-dam-field-hidden');
            $('.ephoto-dam-field-file-button').removeClass('ephoto-dam-field-hidden');
        }
    },

    /**
     * Load Ephoto Dam Api
     */
    load: function() {
        if (typeof ePhoto !== 'undefined') {
            this.init();
        }

        else if (this._settings && this._settings.server_url) {
            $.getScript(this._settings.server_url + '/api/apiJS.js', this.init.bind(this));
        }
    },
    
    /**
     * Init Ephoto Dam Api
     */    
    init: function() {
        var self = this;

        if (!ePhoto) {
            console.log('Error : loading the Ephoto Dam API');
            return;
        }

        this._authid = $.cookie('drupal_ephoto_dam_' + self._name + '_authid');

        this._ephoto = new ePhoto({
            'server': this._settings.server_url,
            'authID': this._authid,
            'client': this.CLIENT_ID
        });

        this._ephoto.connect();

        this._ephoto.File.enableDCore('json');

        this._ephoto.callOnConnect(() => {
            let version = self._ephoto.getVersion().split('.');

            // Requires API version 5.15+ (Ephoto Dam >4.0)
            if (version.length !==3 || (version[0] < 6 && version[1] < 15)) {
                alert(Drupal.t('The version of Ephoto Dam is not compatible with this feature') + ' (' + version + ')');
                return;
            }

            self._authid = self._ephoto.getAuthID();

            $.cookie('drupal_ephoto_dam_' + self._name + '_authid', self._authid, { path:'/' });

            self._connected = true;
        });
    },

    /**
     * Select a file versions
     */
    selectVersions: function() {
        if (!this._getFields()) {
            console.log('Error : field selection error');
            return;
        }

        this._ephoto.File.callOnFileReceived(this.getVersionsUrls.bind(this));

        this._ephoto.File.setButtons(this._ephoto.IMAGE_FILES, [{}]);  

        this._ephoto.File.get({ 'mode' : 'select' });
    },

    /**
     * Get URL of selected file versions
     *
     * @param {null} empty None value
     * @param {array} dcore DublinCore values
     */    
    getVersionsUrls: function(empty, dcore) {
		if (!dcore) {
			return;
		}

        const identifier = parseInt(dcore['dc:identifier:number'], 10);
        const thumbnail = dcore['dc:description.thumbnail'];

        if (!identifier || !thumbnail) {
            console.log('Notice : dublincore values missing');
            return;
        }

        const caption = this._buildCaption(dcore);        

        this._ephoto.File.getVersions(identifier, function(versions) {
            const fields = this._getFields();

            for (const inc in fields) {
                if (!fields[inc].name.length || typeof versions[fields[inc].name] === 'undefined') {
                    fields[inc].input.val('');
                    fields[inc].identifier.val('');
                    fields[inc].caption.val('');
                    fields[inc].thumbnail.val('');

                    fields[inc].thumbnail.prev().attr('src', '');
                    continue;
                }

                let size = this._checkSizeValue(fields[inc].size);
                let version = fields[inc].name;
                let file = version + '@' + identifier;

                let button = this._ephoto.File.setButtons( this._ephoto.IMAGE_FILES, [ {
                    'mode': 'link',
                    'definition': 'custom',
                    'size': size

                } ] ).get(0);

                button.getLink([file], function(url) {
                    fields[inc].input.val(url);
                    fields[inc].identifier.val(identifier);
                    fields[inc].caption.val(caption);
                    fields[inc].thumbnail.val(thumbnail);

                    console.log(identifier);

                    fields[inc].thumbnail.prev().attr('src', thumbnail);
                }); 
            }

        }.bind(this));
    },

    /**
     * Select a file
     * 
     * @param {number} delta Field index
     */    
    selectFile: function(delta) {
        const fields= this._getFields();

        if (!fields.length) {
            console.log('Error : field selection error (1)');
            return;
        }
 
        this._fieldSelected = fields[delta];

        if (!this._fieldSelected) {
            console.log('Error : field selection error (2)');
            return;
        }

        let size = this._checkSizeValue(this._fieldSelected.size);

        // Dimensions are no longer required
        //if (size === '' || size === 0) {
        //    return;
        //}

        this._ephoto.File.callOnFileReceived(this.getFileUrl.bind(this));

        this._ephoto.File.setButtons( this._ephoto.IMAGE_FILES, [ {
            'mode': 'link',
            'definition': 'custom',
            'size': size
        } ] );        

        this._ephoto.File.get();
    },

    /**
     * Get URL of selected file
     * 
     * @param {string} url URL value
     * @param {object} dcore DublinCore values
     */    
    getFileUrl: function(url, dcore) {
        if (url === 'fileDoesNotExist') {
            console.log('Notice : file does not exist');
            return;
        }
		
		if (!dcore) {
			return;
		}

        if (!this._fieldSelected) {
            console.log('Notice : field selected missing');
            return;
        }

        const identifier = parseInt(dcore['dc:identifier:number'], 10);
        const thumbnail = dcore['dc:description.thumbnail'];

        if (!identifier || !thumbnail) {
            console.log('Notice : dublincore values missing');
            return;
        }

        const caption = this._buildCaption(dcore);

        this._fieldSelected.input.val(url);
        this._fieldSelected.identifier.val(identifier);
        this._fieldSelected.caption.val(caption);
        this._fieldSelected.thumbnail.val(thumbnail);
        
        this._fieldSelected.thumbnail.prev().attr('src', thumbnail);
    },

    /**
     * Check size value
     * 
     * @param {number} size A size value
     */    
    _checkSizeValue: function(size) {
        return size.toString().split('x').map((val) => {
            val = val === '' || isNaN(val) ? 0 : parseInt(val, 10);
            return (val > 4000) ? 4000 : ((val <= 0) ? '' : val);

        }).join('x');
    },

    /**
     * Get all fields
     */
    _getFields: function() {
        let fieldname = this._name.replace(/_/g, '-');
        let bloc = $('#' + fieldname + '-values');

        if (!bloc.length) {
            bloc = $('#edit-' + fieldname + '-wrapper');
        }

        let fields = [];
        let sizes = bloc.find('.ephoto-dam-field-imagesize-field').toArray();
        let inputs = bloc.find('.ephoto-dam-field-url-field').toArray();
        let identifiers = bloc.find('.ephoto-dam-field-identifier-field').toArray();
        let captions = bloc.find('.ephoto-dam-field-caption-field').toArray();
        let thumbnails = bloc.find('.ephoto-dam-field-thumbnail').toArray();

        bloc.find('.ephoto-dam-field-version-field').map((index, el) => {
            fields.push({
                'name' : $(el).val(),
                'size' : $(sizes[index]).val(),
                'input' : $(inputs[index]),
                'identifier' : $(identifiers[index]),
                'caption' : $(captions[index]),
                'thumbnail' : $(thumbnails[index])
            });
        });

        return fields;
    },
    
    /**
     * Build the caption width a DublinCore
     * 
     * @param {array} dcore Dublin Core values
     */		
    _buildCaption: function(dcore) {
        let result = this._settings.captions_format;

        for (const property in dcore) {
            result = result.replace('[' + property + ']', dcore[property]);
        }

        return result;
    }
}

EphotoDamField = {};

for (const name in drupalSettings.ephotoDamField) {
    EphotoDamField[name] = new Drupal.ephotoDamField(name, drupalSettings.ephotoDamField[name]);
    EphotoDamField[name].load();
}

})(jQuery);
