/**
 * Ephoto Dam Addon for Drupal 9
 *
 * @base EPHOTO DAM
 * @author EINDEN
 */

(function ($) {
    
    /**
    * A CKEditor plugin for vido embeds.
    */
    CKEDITOR.plugins.add('ephoto_dam', {	
        /**
         * EPHOTO ID of Drupal Addon
         */		
        CLIENT_ID : 'CaCvqS5H',
        
        /**
         * Set the plugin modes.
         */
        modes: {
            wysiwyg: 1
        },

        /**
         * Define the plugin requirements.
         */
        requires: 'widget',

        /**
         * Allow undo actions.
         */
        canUndo: true,
        
        /**
         * Settings
         */		
        settings: null,
        
        /**
         * Ephoto Dam instance
         */		
        ephoto: null,
        
        /**
         * CKEditor instance
         */		
        editor: null,		
        
        /**
         * Ephoto Dam Server connected
         */		
        connected: false,

        /**
         * Init the plugin.
         */
        init: function (editor) {
            let self = this;

            this.settings = editor.config.ephoto_dam;

            this.editor = editor;
            
            if(!editor.ui.addButton || !this.settings.server_url) {
                return;
            }

            CKEDITOR.scriptLoader.load(this.settings.server_url + 'api/apiJS.js', this.initApi.bind(this));
            
            CKEDITOR.dialog.add('ephoto_dam_properties', this.getPropertiesBox);
            
            editor.widgets.add('ephoto_dam', {
                downcast: this.downcast,
                upcast: this.upcast,
                mask: true,
                match: this.match,
                init: function() {
                    this.on('focus', function(event) {
                        let handle = event.sender.element.getChild(1);
                       
						if(!handle) return;

                        if(!handle.hasListeners('mousedown')) {
                            handle.on('mousedown', self.resize.bind(self));
                        }

                        handle.addClass('ephoto_dam_drag_on');
                    });				
                    
                    this.on('blur', function(event) {
                        let handle = event.sender.element.getChild(1);
                        
						if(!handle) return;
    
                        handle.removeClass('ephoto_dam_drag_on');
                    });					
                } 
            });

            editor.addCommand('ephoto_dam_call', {
                exec: this.call.bind(this)
            });
            
            editor.addCommand( 'ephoto_dam_properties', 
                new CKEDITOR.dialogCommand( 'ephoto_dam_properties' )
            );

            editor.ui.addButton('ephoto_dam', {
                label: Drupal.t('Select a file from Ephoto Dam'),
                command: 'ephoto_dam_call',
                icon: this.path + '/icon.png'
            });			
            
            editor.addMenuGroup( 'ephoto_dam_group' );
            
            editor.addMenuItem( 'ephoto_dam_item', {
                label: Drupal.t('Properties'),
                icon: this.path + '/icon.png',
                command: 'ephoto_dam_properties',
                group: 'ephoto_dam_group'
            });			
            
            editor.contextMenu.addListener( function(element) {
                var widget = element.getFirst();
                
                if(!widget || typeof widget.getAttribute !== 'function') return;

                if(widget.getAttribute('data-widget') === 'ephoto_dam') {
                    return { ephoto_dam_item: CKEDITOR.TRISTATE_OFF };
                }
            } );
        },
        
        /**
         * Init API Ephoto Dam
         */		
        initApi: function(success) {
            let self = this;
            
            if(!success) {
                console.log(Drupal.t('Error loading the Ephoto Dam API'));
                return;
            }

            this.ephoto = new ePhoto({
                'server': this.settings.server_url,
                'authID': CKEDITOR.tools.getCookie('drupal_ephoto_dam_authid'),
                'client': this.CLIENT_ID
            });

            this.ephoto.connect();

            this.ephoto.File.setMode('embed');

            this.ephoto.File.enableDCore('json');

            this.ephoto.callOnConnect(function() {
                let version = self.ephoto.getVersion().split('.');

                // Requires API version 5.15 (Ephoto Dam 4.0)
                if (version.length !==3 || (version[0] < 5 && version[0] < 15)) {
                    alert(Drupal.t('The version of Ephoto Dam is not compatible with this feature'));
                    return;
                }

                CKEDITOR.tools.setCookie( 'drupal_ephoto_dam_authid', self.ephoto.getAuthID());

                self.connected = true;
            });				

            this.ephoto.File.callOnFileReceived(this.insert.bind(this));

            this.ephoto.File.setButtons( this.ephoto.IMAGE_FILES, [ {
                'definition': !this.settings.zoom && this.settings.images_size <= 320 ? 'low' : 'middle',
                'size': this.settings.images_size.toString()
            } ] );

            this.ephoto.File.setButtons( this.ephoto.MOVIE_FILES, [ {
                'definition': 'custom',
                'size': this.settings.videos_size.toString()
            } ] );

			/*
            this.ephoto.File.setButtons( this.ephoto.FLASH_FILES, [ {
                'definition': 'custom',
                'size': this.settings.images_size.toString()
            } ] );
			*/

            this.ephoto.File.setButtons( this.ephoto.DOCUMENT_FILES, [ {
                'definition': 'custom',
                'size': this.settings.documents_size.toString()
            } ] );
        },
        
        /**
         * Call Properties Dialog Box
         */		
        getPropertiesBox: function(editor) {
            return {
                title : Drupal.t('Properties'),
                width: 600,
                height: 100,
                contents: [{
                    id: 'properties',
                    elements: [{
                        type: 'text',
                        id: 'alt',
                        label: Drupal.t('Alternative text'),
                        'default': '',
                    } , {
                        type: 'radio',
                        id: 'align',
                        label: Drupal.t('Align'),
                        items: [ [ Drupal.t('None'), 'none' ], [ Drupal.t('Left'), 'left' ], [ Drupal.t('Center'), 'center' ], [ Drupal.t('Right'), 'right' ] ],
                        inputStyle: 'margin-right:8px',
                        style: 'width:80%;margin-left:0',
                        'default': 'none'
                    } , {
                        type: 'checkbox',
                        id: 'caption',
                        label: Drupal.t('Caption'),
                        labelStyle: 'display:inline',
                        'default': true
                    } , {
                        type: 'checkbox',
                        id: 'zoom',
                        label: Drupal.t('Zoom'),
                        labelStyle: 'display:inline',
                        'default': true	
                    }]
                }],
                
                onShow: function() {
                    let widget = editor.widgets.focused;
                    
                    if(!widget || widget.name !== 'ephoto_dam') {
                        return;
                    }

                    this.getContentElement('properties', 'alt').setValue(widget.data.json.alt);
                    this.getContentElement('properties', 'align').setValue(widget.data.json.align);
                    this.getContentElement('properties', 'caption').setValue(widget.data.json.caption);
                    this.getContentElement('properties', 'zoom').setValue(widget.data.json.zoom);
                },
                
                onOk: function() {
                    let widget = editor.widgets.focused;
                    
                    if(!widget || widget.name !== 'ephoto_dam') {
                        return;
                    }

                    widget.removeClass('ephoto_dam_' + widget.data.json.align);
    
                    widget.data.json.alt = this.getContentElement('properties', 'alt').getValue();
                    widget.data.json.align = this.getContentElement('properties', 'align').getValue();
                    widget.data.json.caption = this.getContentElement('properties', 'caption').getValue();
                    widget.data.json.zoom = this.getContentElement('properties', 'zoom').getValue();

                    widget.element.setHtml(
                        Drupal.theme('EphotoDamWidget', widget.data.json)
                    );

                    widget.addClass('ephoto_dam_' + widget.data.json.align);
                }
            };
        },
    
        /**
         * Call API Ephoto Dam
         */
        call: function() {
            if(!this.connected) {
                console.log(Drupal.t('Error loading the Ephoto Dam API'));
                return;
            }

            this.ephoto.File.get();
        },
        
        /**
         * Resize file
         */		
        resize: function(event) {
            event.data.preventDefault();
            event.data.stopPropagation();
            
            let self = this;

            let handle = event.data.getTarget();
            let image = handle.getPrevious();
            let widget = this.editor.widgets.getByElement(image);

            let width = parseInt(image.getAttribute('width'), 10);
            let height = parseInt(image.getAttribute('height'), 10);
            let ratio = height / width;

            let posx = event.data.$.pageX;

            let resizeimage = function(event) {
                event.data.preventDefault();
                event.data.stopPropagation();
                
                let posx2 =  event.data.$.pageX;
                let diff = posx2 - posx;

                if(diff) {
                    let nextwidth = width + diff;
                    let nextheight = Math.floor((width + diff) * ratio);

                    image.setAttribute('width', nextwidth);
                    image.setAttribute('height', nextheight);
                    
                    handle.setStyle('margin-left', (nextwidth - 3) + 'px');
                    
                    widget.data.json.width = nextwidth;
                    widget.data.json.height = nextheight;
                    
                    widget.data.json.embed = self.updateURLParam(widget.data.json.embed, 'width', nextwidth);
                    widget.data.json.embed = self.updateURLParam(widget.data.json.embed, 'height', nextheight);					
                }
            }
            
            let document = CKEDITOR.currentInstance.document;

            document.on('mousemove', resizeimage);

            document.on('mouseup', function (event) {
                event.data.preventDefault();
                event.data.stopPropagation();
                
                event.removeListener();
                
                document.removeListener('mousemove', resizeimage);
            });		
        },
        
        /**
         * Insert a selected file
         */		
        insert: function(html, dcore) {
            if(!html || !dcore) {
                return;
            }

            let self = this;
            let embed = this.getURLFromString(html, 'embed');
            
            if(!embed) return;

            $.ajax({
                url : embed
            
            }).done(function(data) {
                let dctype = dcore['dc:type'];
                let src, size, type;
                
                switch(dctype) {
                    case 'Image':
                        type = 'image';
                        src = self.getURLFromString(data, 'pubpreview');
                        embed = '';
                        size = self.settings.images_size;
                        break;

                    case 'MovingImage':
                    case 'Sound':
                        type = 'video';
                        src = self.getURLFromString(data, 'pubscreenshot');
                        size = self.settings.videos_size;
                        break;

                    case 'Dataset':
                        type = 'book';
                        src = self.getURLFromString(data, 'pubbook') + '0';
                        size = self.settings.documents_size;
                        break;
                }

                self.createWidget(type, src, embed, size, dcore);
            });
        },
        
        /**
         *
         */		
        createWidget: function(type, src, embed, size, dcore) {
            let self = this;

            // Load a cache of the image to detect width and height
            let cache = new Image();

            cache.onload = function() {
                let widget, width, height, ratio = cache.width / cache.height;

                if (ratio >= 1.0 && type !== 'video') {
                    width = size;
                    height = Math.round(size * (1 / ratio));
                }
				
				else {
                    height = size;
                    width = Math.round(size * ratio);
                }
                
                if(type === 'book') width *= 2;
                
                embed = self.updateURLParam(embed, 'width', width);
                embed = self.updateURLParam(embed, 'height', height);

                // Redimensionnement : display span drag : inline-block
                widget = self.editor.document.createElement('p');

                widget.setHtml(JSON.stringify({
                    'ephoto_dam': type,
                    'align': 'none',
                    'src': src,
                    'embed': embed,
                    'width': width,
                    'height': height,
                    'alt': '',
                    'caption': self.settings.captions,
                    'caption_data': self.buildCaption(dcore),
                    'zoom': self.settings.zoom,
                    'identifier': parseInt(dcore['dc:identifier:number'], 10)
                }));

                self.editor.insertHtml(widget.getOuterHtml());
            };

            cache.src = src;	
        },
        
        /**
         *
         */		
        match: function(element) {
            return element.getHtml().match(/^({(?=.*ephoto_dam\b)(?=.*align\b)(?=.*src\b)(?=.*embed\b)(?=.*width\b)(?=.*height\b)(?=.*alt\b)(?=.*caption\b)(?=.*caption_data\b)(?=.*zoom\b)(.*)})$/);
        },

        /**
         * Check if the element is an instance of the Ephoto Dam widget
         */
        upcast: function(element, data) {
            if (!this.match(element)) {
                return;
            }

            data.json = JSON.parse(element.getHtml());

            let widget = Drupal.theme('EphotoDamWidget', data.json);

            element.setHtml(widget);
            
            element.addClass('ephoto_dam_' + data.json.align);
            
            element.addClass('ephoto_dam_' + data.json.ephoto_dam);

            return element;
        },

        /**
         * Turns a transformed widget into the downcasted representation
         */
        downcast: function(element) {
            element.setHtml(JSON.stringify(this.data.json));
            
            element.removeClass('ephoto_dam_' + this.data.json.align);
            
            element.removeClass('ephoto_dam_' + this.data.json.ephoto_dam);	
        },

        /**
         * Build the caption width a DublinCore
         */		
        buildCaption: function(dcore) {
            let result = this.settings.captions_format;

            for (const property in dcore) {
                result = result.replace('[' + property + ']', dcore[property]); 
            }

            return result;
        },
        
        /**
         *
         */		
        updateURLParam: function(url, param, paramVal) {
            let newAdditionalURL = '';
            let tempArray = url.split('?');
            let baseURL = tempArray[0];
            let additionalURL = tempArray[1];
            let temp = '';
            
            if (additionalURL) {
                tempArray = additionalURL.split('&');
            
                for (let i = 0; i < tempArray.length; i++) {
                    if(tempArray[i].split('=')[0] != param){
                        newAdditionalURL += temp + tempArray[i];
                        temp = '&';
                    }
                }
            }

            let rows_txt = temp + param + '=' + paramVal;
            
            return baseURL + '?' + newAdditionalURL + rows_txt;
        },		
        
        /**
         *
         */		
        getURLFromString: function(str, filter) {
            let re = /((([A-Za-z]{3,9}:(?:\/\/)?)(?:[-;:&=+$,\w]+@)?[A-Za-z0-9.-]+|(?:www\.|[-;:&=+$,\w]+@)[A-Za-z0-9.-]+)((?:\/[+~%\/.\w-]*)?\??(?:[-+=&;%@.\w]*)#?\w*)?)/gm;
            let m, result = null;
            
            while ((m = re.exec(str)) !== null) {
                if (m.index === re.lastIndex) {
                    re.lastIndex++;
                }
                
                if(m[0].indexOf(filter) !== -1) {
                    result = m[0];
                    break;
                }
            }
 
            return result;
        }
    });
    
    /**
    * The widget template viewable in the WYSIWYG
    */
    Drupal.theme.EphotoDamWidget = function (settings) {
        let html;

        let type = settings.ephoto_dam;
        let src = Drupal.checkPlain(settings.src);
        let width = settings.width;
        let height = settings.height;
        let alt = Drupal.checkPlain(settings.alt);
        let caption = settings.caption;
        let caption_data = Drupal.checkPlain(settings.caption_data);
        let identifier = settings.identifier;

        let handle = width - 3;
        
        let bookstyle = '';
        
        if(type === 'book') {
            width = Math.floor(width / 2);
            let margin = Math.floor(width / 2);
            bookstyle = ' ephoto_dam_book_bck" style="padding-left:' + margin + 'px;padding-right:' + margin + 'px';
        }

        html =[
            '<img src="' + src + '" alt="' + alt + '" width="' + width + '" height="' + height + '" class="ephoto_dam_file' + bookstyle + '" data-identifier="' + identifier + '">',
            '<span class="ephoto_dam_drag" title="' + Drupal.t('Click and drag to resize') + '" style="margin-left:' + handle + 'px">​</span>',
            caption ? '<span class="ephoto_dam_caption">' + caption_data + '</span>' : ''
        ].join('');
        
        return html;
    };	

})(jQuery);
